from stub_generator.stub_generator.rtype._function_mapping import update_function_rtype
from stub_generator.stub_generator.rtype._method_mapping import update_method_rtype
from stub_generator.stub_generator.rtype._property_mapping import update_property_rtype
