from focs._tech import *

Tech(
    name="TECH_COL_MISIORLA",
    description="TECH_COL_MISIORLA_DESC",
    short_description="EXTINCT_UNLOCK_SHORT_DESC",
    category="SPY_CATEGORY",
    researchcost=1,
    researchturns=1,
    researchable=False,
    tags=["PEDIA_EXTINCT_SPECIES_TECHS", "SP_MISIORLA", "CTRL_EXTINCT"],
    graphic="icons/species/misiorla.png",
)
